
/*****************************************************************************
 *
 *  xinout.h
 *
 *  $Log$
 *
 *	Written by Shaun Terry, Copyright 1995
 *
 *****************************************************************************
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/types.h>

#define VERSION		"Xinout 1.1"


typedef int bool;
#ifndef FALSE
#define FALSE	0
#define TRUE	1
#endif

#ifndef SERVER

#define IN_FLAG			1
#define OUT_FLAG		2

#define MAXCOMM			500
#define MAXPROJECTS		6

#define MAXWINDOWWIDTH	800

#define MAXPEOPLE		100

struct PeopleInfo {
    char *Name;
    char *FullName;
    char *OfficePhone;
    char *HomePhone;
    char *BeeperNum;
    char *Email;
};

enum {
	COMMENTLABEL=10, COMMENTTEXT, GROUPMSG, COMMANDPOPUP
};

enum {
	DTLS_NAME=20, DTLS_OPHONE, DTLS_HPHONE, DTLS_BEEPNUM, DTLS_EMAIL
};

enum {
	GMSGFLD=40
};

#define WHOCOL		100
#define INCOL		200
#define OUTCOL		300
#define COMMSCOL	400

struct PopupButtonInfo {
	char *name;
	void (*callback)(Widget, XtPointer, XtPointer);
	caddr_t clientData;
};

// Function prototypes
void createWidgets(Widget);
void CreateBulletPixmap(Widget, Pixmap *);
void CreateSquarePixmap(Widget, Pixmap *);
void CreateBlankPixmap(Widget, Pixmap *);
void CreateArrowPixmap(Widget, Pixmap *);
void FreePixmaps();
void evPerson(Widget, caddr_t, XEvent *, char *);
void evComment(Widget, caddr_t, XEvent *, char *);
void evMenuPopup(Widget, caddr_t, XEvent *, char *);
void cbAbout(Widget, caddr_t, caddr_t);
void cbIn(Widget, caddr_t, caddr_t);
void cbOut(Widget, caddr_t, caddr_t);
void cbReasons(Widget, caddr_t, caddr_t);
void cbCommentApply(Widget, caddr_t, caddr_t);
void cbSetGMsg(Widget, caddr_t, caddr_t);
void cbGMApply(Widget, caddr_t, caddr_t);
void cbFortune(Widget, caddr_t, caddr_t);
void cbPopupDismiss(Widget, XtPointer, XtPointer);
void cbEndit();
void cbRead(XtPointer, int *, XtInputId *);
void change_status(int, char, bool);
void change_comment(int, char *, bool);
void change_grpmsg(char, char *, bool);
void change_fortune(char, char *, bool);
void display_details(int);
void info_request();
int intwho(char *);
void send_info();
void readConfigFile(char *);
Widget createPopupWindow(char *, Widget, PopupButtonInfo *, Widget *, Widget *);
void restrictSize(int minH, int minW, int maxH, int maxW);

#define SetToCurrent		0
#define SetToUnrestricted	(-1)
#define LeaveUnchanged		(-2)

void setLabelString(Widget, char *);
void setLabelString(int, char *);
void setMessageString(Widget, char *);
char *getLabelString(Widget);
char *getLabelString(int);

inline XmFontList getFontList(Display *dpy, char *fontName) {
	XFontStruct *fnt = XLoadQueryFont(dpy, fontName);
	return XmFontListCreate(fnt, XmSTRING_DEFAULT_CHARSET);
}

inline XmString stringCreate(char *str) {
	return XmStringCreateLtoR(str, XmSTRING_DEFAULT_CHARSET);
}

inline void stringFree(XmString str) { XmStringFree(str); }

inline void setIconName(Widget top, char *name) {
	XTextProperty iconName;
	XStringListToTextProperty(&name, 1, &iconName);
	XSetWMIconName(XtDisplay(top), XtWindow(top), &iconName);
}

inline void setTitle(Widget top, char *title)
{
    XTextProperty windowName;

    XStringListToTextProperty(&title, 1, &windowName);
    XSetWMName(XtDisplay(top), XtWindow(top), &windowName);
}

inline isButton1(XButtonPressedEvent *e) { return (e->button & 0xff)==Button1; }
inline isButton2(XButtonPressedEvent *e) { return (e->button & 0xff)==Button2; }
inline isButton3(XButtonPressedEvent *e) { return (e->button & 0xff)==Button3; }

inline char *skipWhiteSpace(char *&p)
{
    while ((*p == ' ' || *p == '\t') && *p != '\0') ++p;
    return p;
}

inline char *skipWhiteSpaceBackward(char *&p, char *loBound)
{
    if (p < loBound) return (p = loBound);

    while (*p == ' ' || *p == '\t') --p;

    return (p > loBound) ? p : (p = loBound);
}

class ConfigFile {

private:
	char fileName[256];
	FILE *fp;
	bool inReadSection, newSection;
	bool fExists;

	char *normalizeString(char *, bool =TRUE);
	char *denormalizeString(char *p, bool =TRUE);

	char *nextLine();

public:
	ConfigFile(char *name);
	~ConfigFile() {
		close(); 
	}

	bool seekSection(char *section);
	char *readNext();

	char *readField(char *);
	char *readField(char *, int *);
	char *readField(char *f, int &v) { return readField(f, &v); }
	char *readField(char *, long *);
	char *readField(char *f, long &v) { return readField(f, &v); }
	char *readField(char *, double *);
	char *readField(char *f, double &v) { return readField(f, &v); }

	char *readString(char *field, char *dflt);
	int readInt(char *field, int dflt);
	double readFloat(char *field, double dflt);

	bool open();
	void close();

	char const *name() { return fileName; } 
	bool exists() { return fExists; }
};

#endif	// #ifndef SERVER

bool getParamInt(int, char **, char *, int &);
bool getParamString(int, char **, char *, char *&);
bool getParamFlag(int, char **, char *);

inline bool getParamString(int argc, char **argv, char *opt, char *&param)
{
	for (int i=1; i < argc; ++i) {
		if (!strcmp(opt, argv[i])) {
			if (i == argc-1) return FALSE;
			param = argv[i+1];
			return TRUE;
		}
	}
	return FALSE;
}

inline bool getParamFlag(int argc, char **argv, char *opt)
{
	for (int i=1; i < argc; ++i) {
		if (!strcmp(opt, argv[i])) return TRUE;
	}

	return FALSE;
}

inline bool getParamInt(int argc, char **argv, char *opt, int &i)
{
	char *p;

	if (getParamString(argc, argv, opt, p)) {
		i = atoi(p);
		return TRUE;
	}
	else return FALSE;
}

enum MSG_TYPE {
	STATUS_CHANGE,
	COMMENT_CHANGE,
	INFO_REQUEST,
	SET_GROUP_MSG,
	REMOVE_GROUP_MSG,
	SET_FORTUNE,
	REMOVE_FORTUNE
};

struct inout_msg_hdr {
    MSG_TYPE type;
    char who[256];
    char in_or_out;
	int textlen;
};

int sendToServer(MSG_TYPE, char *, char, char * ="");
int connectToServer();

